/*
 * hmi.cpp
 *
 *  Created on: 22.02.2015
 *      Author: wn
 */

#include <msp430g2553.h>
#include <isr_compat.h>
#include <stdint.h>

#include "hmi.h"
#include "pwm.h"
#include "mySpi.h"
#include "display.h"




class SwitchPort2 {
public:
	SwitchPort2(uint16_t bit) : m_bit(bit), m_state(0), m_cnt(0) {
		P2REN |= m_bit;
	};
	bool get() {
		bool result = false;
		switch (m_state) {
		case 0:
			if (P2IN & m_bit) {
				m_cnt = 0;
			} else {
				m_cnt++;
			}
			if (m_cnt > 100) {
				m_state = 1;
				m_cnt = 0;
			}
			break;
		case 1:
			result = true;
			m_state = 2;
			break;
		case 2:
			if (P2IN & m_bit) {
				m_cnt++;
			} else {
				m_cnt = 0;
			}
			if (m_cnt > 20) {
				m_state = 0;
				m_cnt = 0;
			}
			break;
		default:
			m_state = 0;
			break;
		}
		return result;
	};
private:
	uint16_t m_bit;
	uint8_t m_state;
	uint16_t m_cnt;
};



SwitchPort2 switch1 = SwitchPort2(BIT0);
SwitchPort2 switch2 = SwitchPort2(BIT1);
SwitchPort2 switch3 = SwitchPort2(BIT2);
SwitchPort2 switch4 = SwitchPort2(BIT3);


const uint8_t LED_RED_PIN = BIT5;
const uint8_t LED_GREEN_PIN = BIT4;

typedef enum { e_OFF, e_RED, e_GREEN } LedColor;

void hmiInit() {
	P1DIR |= LED_RED_PIN | LED_GREEN_PIN;
	P1OUT |= LED_RED_PIN | LED_GREEN_PIN;
	spiInit();
	dispInit();
}

void setLed(LedColor ledColor) {
	switch (ledColor) {
	case e_GREEN:
		P1OUT |= LED_RED_PIN;
		P1OUT &= ~LED_GREEN_PIN;
		break;
	case e_RED:
		P1OUT |= LED_GREEN_PIN;
		P1OUT &= ~LED_RED_PIN;
		break;
	case e_OFF:
	default:
		P1OUT |= LED_RED_PIN | LED_GREEN_PIN;
		break;
	}
}


void hmiExec() {
	static float savedVoltage = 0.0;
	static bool enabled = false;
	static uint8_t dispState = 0;
	const uint8_t MAX_DISP_STATE = 1;

	if (switch1.get()) {
		if (enabled) {
			enabled = false;
			setLed(e_OFF);
			setUDes(0);
		} else {
			enabled = true;
			setLed(e_GREEN);
			setUDes(savedVoltage);
		}
	}
	if (switch2.get()) {
		savedVoltage -= 1.0;
		if (enabled) {
			setUDes(savedVoltage);
		}
	}
	if (switch3.get()) {
		savedVoltage += 1.0;
		if (enabled) {
			setUDes(savedVoltage);
		}
	}
//	if (switch4.get()) {
//		dispState++;
//		if (dispState > MAX_DISP_STATE) {
//			dispState = 0;
//		}
//	}

	static uint32_t cnt = 0;
	cnt++;
	if (cnt >= 10000) {
		dispSetFloat(0, getUCur());
		float dispValue;
		switch (dispState) {
		case 0:
			dispValue = savedVoltage;
			break;
		case 1:
			dispValue = getDutyCycle();
			break;
		default:
			dispValue = 99999; // illegal value
		}
		dispSetFloat(1, dispValue);
		cnt = 0;
	}
}
